/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
var canvas  = document.getElementById('wrappingpaper');
var context = canvas.getContext('2d');
canvas.focus();

//globals    
var gMouseX     = -1;
var gMouseY     = 0;
var gMouseClick = false;
var gFPS        = 30;
var gThink		= 100;

//mouse handler - in place definition
canvas.onmousemove = function OnMouseMove( event ){
	if (gMouseClick) {
		gMouseX = event.clientX - canvas.offsetLeft;
		gMouseY = event.clientY - canvas.offsetTop;
	} else {
		gMouseX = -1;
	}
};
canvas.onmousedown = function OnMouseDown( event ){
	gMouseX = event.clientX - canvas.offsetLeft;
    gMouseY = event.clientY - canvas.offsetTop;
    gMouseClick = true;
};
canvas.onmouseup = function OnMouseUp( event ){
    gMouseClick = false;
	gMouseX = -1;
};

//touch event handler
canvas.ontouchmove = function OnTouchMove( event ){
	event.stopPropagation();
	event.preventDefault();
	if (gMouseClick) {
		gMouseX = event.touches[0].clientX - canvas.offsetLeft;
		gMouseY = event.touches[0].clientY - canvas.offsetTop;
	} else {
		gMouseX = -1;
	}
};
canvas.ontouchstart = function OnTouchStart( event ){
	event.stopPropagation();
	event.preventDefault();
    gMouseClick = true;
	gMouseX = event.touches[0].clientX - canvas.offsetLeft;
	gMouseY = event.touches[0].clientY - canvas.offsetTop;
};
canvas.ontouchend = function OnTouchEnd( event ){
	event.stopPropagation();
	event.preventDefault();
    gMouseClick = false;
	gMouseX = -1;
};

var levels = 
				'100000000' +
				'022200000' +
				'000020000' +
				'000020000' +
				'000002220' +
				'000000002' +
				// level 2
				'000004000' +
				'044444440' +
				'000004004' +
				'000004004' +
				'000004004' +
				'300000440' +
				// level 3
				'060000006' +
				'600000000' +
				'000000006' +
				'000000006' +
				'000000000' +
				'000000005' +
				// level 4
				'020002002' +
				'202020202' +
				'202020202' +
				'022222002' +
				'002020002' +
				'001002220'+
				// level 5
				'020042003' +
				'202402400' +
				'202402404' +
				'202402404' +
				'002402404' +
				'100420040' +
				// level 6
				'000000000' +
				'000600000' +
				'506460000' +
				'000600000' +
				'000000000' +
				'000300000' +
				// level
				'000660260' +
				'006006000' +
				'020200200' +
				'006006060' +
				'000200200' +
				'015022000' +
				// level 7
				'044444440' +
				'404444404' +
				'440444044' +
				'444444044' +
				'440444404' +
				'304444440'+
				// level
				'300600000' +
				'000000400' +
				'006004640' +
				'064600400' +
				'006000000' +
				'000004005' +
				// level 8
				'000000200' +
				'044442420' +
				'402002024' +
				'000262624' +
				'000202624' +
				'513020624' +
				// level
				'000888605' +
				'008000680' +
				'008666680' +
				'068888600' +
				'068000600' +
				'708666000' +
				// level 9
				'000200000' +
				'200004000' +
				'060020420' +
				'006400000' +
				'000606000' +
				'350610420' +
				// level 10
				'006060008' +
				'700006400' +
				'108400000' +
				'300020806' +
				'500800004' +
				'000002002' +
				// level 11
				'300000000' +
				'700004400' +
				'180084482' +
				'082280084' +
				'002200008' +
				'000000000' +
				// level
				'404540400' +
				'006066040' +
				'060040640' +
				'064060040' +
				'060440400' +
				'006063606' +
				// level
				'022210600' +
				'206666660' +
				'260222006' +
				'200666026' +
				'022222206' +
				'002056660' +
				// level 12
				'000246000' +
				'006246600' +
				'046246640' +
				'246046640' +
				'000006600' +
				'135000000' +
				// level
				'200004260' +
				'000000002' +
				'004000000' +
				'000000040' +
				'000200000' +
				'513000000' +
				// level
				'000000005' +
				'880060000' +
				'084680660' +
				'000004000' +
				'004000608' +
				'370000040' +
				// level
				'008000000' +
				'404800808' +
				'088040880' +
				'000800000' +
				'004044008' +
				'300700080' +
				// level
				'024000400' +
				'000204240' +
				'024224240' +
				'000000000' +
				'024200440' +
				'310000000' +
				// level
				'000000000' +
				'706600668' +
				'086688660' +
				'086688660' +
				'580088006' +
				'000000000' +
				// level
				'300444000' +
				'000666460' +
				'006444464' +
				'646666400' +
				'046444000' +
				'000666005' +
				// level
				'000000200' +
				'000008080' +
				'000020000' +
				'000800202' +
				'002008000' +
				'170000800' +
				// level
				'000600305' +
				'262000000' +
				'000064464' +
				'262260000' +
				'000000464' +
				'601006000' +
				// level
				'000000000' +
				'002400006' +
				'000024600' +
				'040000004' +
				'000600400' +
				'000513062' +
				// level
				'000040000' +
				'500060440' +
				'466000000' +
				'000000446' +
				'066040003' +
				'000060000' +
				// level
				'002226000' +
				'526666200' +
				'026660060' +
				'020022260' +
				'006222261' +
				'000266600' +
				// level
				'800705004' +
				'060086600' +
				'000406004' +
				'408640080' +
				'000404680' +
				'000038000' +
				// level
				'100020003' +
				'042040660' +
				'000000002' +
				'082280606' +
				'000200000' +
				'780004065' +
				// level
				'000000062' +
				'000000060' +
				'000066200' +
				'000622000' +
				'002000000' +
				'150000000' +
				// level
				'204000035' +
				'000000000' +
				'000062840' +
				'084620000' +
				'000000000' +
				'170000806' +
				// level 13
				'060600600' +
				'000006260' +
				'000000605' +
				'000020001' +
				'000262000' +
				'020020020' +
				// level 14
				'800000800' +
				'006068000' +
				'040000608' +
				'000400860' +
				'000000400' +
				'600357004' +
				// level 15
				'000000020' +
				'500600000' +
				'020620020' +
				'006206006' +
				'100200000' +
				'000000006' +
				// level 16
				'000000000' +
				'040000040' +
				'600040006' +
				'006464600' +
				'000000000' +
				'005403600' +
				// level 17
				'800007000' +
				'000400400' +
				'008040080' +
				'040080400' +
				'008008000' +
				'000300004' +
				// level
				'100020003' +
				'000020420' +
				'402440000' +
				'000042402' +
				'400200020' +
				'040402000' +
				// level 19
				'003000400' +
				'020242020' +
				'000222000' +
				'404000404' +
				'020000020' +
				'010000020' +
				// level 20
				'060000208' +
				'660808000' +
				'002008068' +
				'000000000' +
				'000000000' +
				'100050007'
				;

var colors = [
	'rgb(90, 160, 230)',
	'rgb(60, 100 ,140)',
	'rgb(70, 140, 70)',
	'rgb(50, 90, 50)',
	'rgb(180, 60, 30)',
	'rgb(120, 50, 30)',
	'rgb(200, 160, 60)',
	'rgb(140, 120, 60)'
];
		
var s = '';
var w = 0;
var x = 0;
var y = 0;
var j = 0;
var i = 0;
var a = -1;
var b = -1;
var lastA = -1;
var lastB = -1;
var curLayer = 0;
var curLevel = 0;
var dragCur = -1;

var level = createArray(18, 9);

level[0][0] = -1;
var bWin = false;
var bLastCheck = false;
var bRender = false;

// erstes Level laden
GameLoop();

//Draw Background
Render();

//Sokobond game loop - every 10 ms
setInterval(GameLoop, 1000/gThink);

//Sokobond Rendering loop
//setInterval(Render, 1000/gFPS);

function createArray(length) {
    var arr = new Array(length || 0),
        i = length;

    if (arguments.length > 1) {
        var args = Array.prototype.slice.call(arguments, 1);
        while(i--) arr[length-1 - i] = createArray.apply(this, args);
    }

    return arr;
}

function DrawBackground(){
    // Hintergrund malen
    context.fillStyle='rgb(255, 255, 255)';
	context.fillRect(0, 0, 576, 64);
	context.fillRect(0, 448, 576, 32);
	
	for (j = 0; j < level.length/3; j++) {
		for (i = 0; i < level[0].length; i++) {
			context.fillStyle='rgb(240, 240, 240)';
			if ((i+j)%2 == 0) {
				context.fillStyle='rgb(220, 220, 220)';	
			}
			context.fillRect(i * 64, (j+1) * 64, 64, 64);
		}
	}
}

function GameLoop(){
	if (level[0][0] == -1) {
		bWin = false;
		bLastCheck = true;
		bRender = true;
		a = b = gMouseY = gMouseX = -1;
		s = levels;
		for (y = 0; y < 6; y++) {
			for (x = 0; x < 9; x++) {
				level[y][x] = parseInt(s.substring(curLevel * 54 + x + y * 9, curLevel * 54 + x + y * 9 + 1));
				level[y+6][x] = level[y+12][x] = 0;
				if (level[y][x]%2 != 0) {
					level[y+12][x] = Math.floor(((Math.floor)(level[y][x]/2)+1) * 10);
					if ((y == 0) || (y == 5)) {
						level[y+6][x] = level[y+12][x];
						level[y+12][x] = 0;
					}								
				}
			}
		}
	}
	if (!bWin) {
		if (dragCur <= 0) {
			if (gMouseY > 448) {
				if ((gMouseX > 272) && (gMouseX < 304)) {
					level[0][0] = -1;
				}
				if ((gMouseX > 232) && (gMouseX < 264)) {
					level[0][0] = -1;
					curLevel -= 1;
					if (curLevel < 0) {
						curLevel = levels.length / 54 - 1;
					}
				}
				if ((gMouseX > 312) && (gMouseX < 344)) {
					level[0][0] = -1;
					curLevel += 1;
					if (curLevel >= levels.length / 54) {
						curLevel = 0;
					}
				}
			} 
			if ((gMouseY < 448) && (gMouseY > 64) && (gMouseX < 576) && (gMouseX >= 0)) {
				x = Math.floor(gMouseX / 64);
				y = Math.floor((gMouseY-64) / 64);
				dragCur = level[y+6][x];
				curLayer = 6;
				lastA = lastB = -1;

				// count the neighbours
				w = 0;
				if (dragCur != 0) {
					if (dragCur%10 == 0) {
						if ((y - 1 >= 0) && ((Math.floor(level[y+6-1][x]/10) == Math.floor(dragCur/10)) && ((level[y+6-1][x] % 10 == 1) || (level[y+6-1][x] % 10 == 4) || (level[y+6-1][x] % 10 == 0)))) {
							w += 1;
						}
						if ((y < 5) && ((Math.floor(level[y+6+1][x]/10) == Math.floor(dragCur/10)) && ((level[y+6+1][x] % 10 == 2) || (level[y+6+1][x] % 10 == 3) || (level[y+6+1][x] % 10 == 0)))) {
							w += 1;
						}
					}
					if (dragCur%10 == 1) {
						if ((x - 1 >= 0) && ((Math.floor(level[y+12][x-1]/10) == Math.floor(dragCur/10)) || ((Math.floor(level[y+6][x-1]/10) == Math.floor(dragCur/10)) && ((level[y+6][x-1] % 10 == 3) || (level[y+6][x-1] % 10 == 4))))) {
							w += 1;
						}
						if ((y < 5) && ((Math.floor(level[y+6+1][x]/10) == Math.floor(dragCur/10)) && ((level[y+6+1][x] % 10 == 2) || (level[y+6+1][x] % 10 == 3) || (level[y+6+1][x] % 10 == 0)))) {
							w += 1;
						}
					}
					if (dragCur%10 == 2) {
						if ((x - 1 >= 0) && ((Math.floor(level[y+12][x-1]/10) == Math.floor(dragCur/10)) || ((Math.floor(level[y+6][x-1]/10) == Math.floor(dragCur/10)) && ((level[y+6][x-1] % 10 == 3) || (level[y+6][x-1] % 10 == 4))))) {
							w += 1;
						}
						if ((y - 1 >= 0) && ((Math.floor(level[y+6-1][x]/10) == Math.floor(dragCur/10)) && ((level[y+6-1][x] % 10 == 1) || (level[y+6-1][x] % 10 == 4) || (level[y+6-1][x] % 10 == 0)))) {
							w += 1;
						}
					}
					if (dragCur%10 == 3) {
						if ((x + 1 <= 8) && ((Math.floor(level[y+12][x+1]/10) == Math.floor(dragCur/10)) || ((Math.floor(level[y+6][x+1]/10) == Math.floor(dragCur/10)) && ((level[y+6][x+1] % 10 == 1) || (level[y+6][x+1] % 10 == 2))))) {
							w += 1;
						}
						if ((y - 1 >= 0) && ((Math.floor(level[y+6-1][x]/10) == Math.floor(dragCur/10)) && ((level[y+6-1][x] % 10 == 1) || (level[y+6-1][x] % 10 == 4) || (level[y+6-1][x] % 10 == 0)))) {
							w += 1;
						}
					}
					if (dragCur%10 == 4) {
						if ((x + 1 <= 8) && ((Math.floor(level[y+12][x+1]/10) == Math.floor(dragCur/10)) || ((Math.floor(level[y+6][x+1]/10) == Math.floor(dragCur/10)) && ((level[y+6][x+1] % 10 == 1) || (level[y+6][x+1] % 10 == 2))))) {
							w += 1;
						}
						if ((y < 5) && ((Math.floor(level[y+6+1][x]/10) == Math.floor(dragCur/10)) && ((level[y+6+1][x] % 10 == 2) || (level[y+6+1][x] % 10 == 3) || (level[y+6+1][x] % 10 == 0)))) {
							w += 1;
						}
					}
					
					
					if (w >= 2) {
						w = dragCur = 0;
					}
					if ((w == 0) && (level[y][x]%2 == 0)) {
						w = dragCur = 0;
					}
					if ((w == 1) && (((Math.floor(dragCur/10))*2-1) == level[y][x])) {
						w = dragCur = 0;
					}
				}
				if (dragCur == 0) {
					dragCur = level[y+12][x];
					curLayer = 12;
					if (dragCur > 0) {
						if ((x - 1 >= 0) && ((Math.floor(level[y+12][x-1]/10) == Math.floor(dragCur/10)) || ((Math.floor(level[y+6][x-1]/10) == Math.floor(dragCur/10)) && ((level[y+6][x-1] % 10 == 3) || (level[y+6][x-1] % 10 == 4))))) {
							w += 1;
						}
						if ((x + 1 <= 8) && ((Math.floor(level[y+12][x+1]/10) == Math.floor(dragCur/10)) || ((Math.floor(level[y+6][x+1]/10) == Math.floor(dragCur/10)) && ((level[y+6][x+1] % 10 == 1) || (level[y+6][x+1] % 10 == 2))))) {
							w += 1;
						}
						if (w == 2) {
							dragCur = -1;
						}
						if ((w === 0) && (level[y][x]%2 === 0)) {
							dragCur = -1;
						}
					}
				}
				if (dragCur > 0) {
					bLastCheck = true;
					a = x;
					b = y;
					dragCur = 10 * (Math.floor)(dragCur / 10);
					if (curLayer == 12) {
						lastA = a + 1;
						if ((x == 8) || ((x >= 1) && ((level[y+12][x-1] == dragCur) || (level[y+6][x-1]%10 == 0)))) {
							lastA = a - 1;	
						}
					}
				}
			}
		} else {
			if ((gMouseY < 448) && (gMouseY > 64) && (gMouseX >= 0) && (gMouseX < 576) && (a >= 0)) {
				x = Math.floor(gMouseX / 64);
				y = Math.floor((gMouseY-64) / 64);
				// check left / right
				if ((Math.abs(x - a) == 1) && (y == b) && ((level[y+6][x]%10 == 0) || (Math.floor(level[y+6][x]/10) == Math.floor(dragCur/10)))) {
					if (((Math.floor(level[y+12][x]/10) != Math.floor(dragCur/10)) && (level[y+12][x] > 0)) || ((curLayer == 6) && ((level[y+12][a] > 0) || ((level[y+6][x] > 0) && (level[y+6][x]%10 != 0)))) || ((curLayer == 12) && (Math.floor(level[y+6][x]/10) == Math.floor(dragCur/10)) && (level[y+6][x]%10 != 0) && (lastA != x))) {
					} else {
						bLastCheck = true;
						if ((lastA == x) && ((level[y+12][x] == dragCur) || ((Math.floor(level[y+6][x]/10) == Math.floor(dragCur/10)) && (level[y+6][x]%10 != 0)))) {

							if ((Math.floor(level[y+6][a]/10) == Math.floor(dragCur/10)) && (level[y+12][a] != dragCur)) {
								level[y+6][a] = 0;
							} else {
								level[y+12][a] = 0;
								curLayer = 12;
							}
						} else {
							if (curLayer == 6) {
								if (x < a) {
									if (((lastB >= 0) && (lastB < b)) || ((lastB < 0) && (y == 0))) {
										level[y+6][a] = dragCur + 2;	
									}
									if (((lastB >= 0) && (lastB > b)) || ((lastB < 0) && (y > 0))) {
										level[y+6][a] = dragCur + 1;	
									}
								}
								if (x > a) {
									if (((lastB >= 0) && (lastB < b)) || ((lastB < 0) && (y == 0))) {
										level[y+6][a] = dragCur + 3;	
									}
									if (((lastB >= 0) && (lastB > b)) || ((lastB < 0) && (y > 0))) {
										level[y+6][a] = dragCur + 4;	
									}
								}
							}
							level[y+12][x] = dragCur;
						}

						curLayer = 12;
						a = x;
						if ((Math.floor(level[y+6][x]/10) == Math.floor(dragCur/10)) && (level[y+12][x] != dragCur)) {
							curLayer = 6;
							level[y+6][x] = dragCur;
						}
					}
				}
				// check up / down
				if ((Math.abs(y - b) == 1) && (x == a)) {
					if (((Math.floor(level[y+6][x]/10) != Math.floor(dragCur/10)) && (level[y+6][x] > 0)) || ((curLayer == 12) && (level[y+6][x] > 0))) {
					} else if ((curLayer == 6) && (Math.floor(level[y+6][x]/10) == Math.floor(dragCur/10)) && (level[y+6][x]%10 != 0) && (lastB != y)) {
					} else {
						bLastCheck = true;
						if ((lastB == y) && (Math.floor(level[y+6][x]/10) == Math.floor(dragCur/10))) {
							level[b+6][a] = 0;
						} else {
							if (curLayer == 12) {
								level[b+12][a] = 0;
								if (y < b) {
									if (((lastA >= 0) && (lastA < a)) || ((lastA < 0) && (x == 0))) {
										level[b+6][a] = dragCur + 2;	
									}
									if (((lastA >= 0) && (lastA > a)) || ((lastA < 0) && (x > 0))) {
										level[b+6][a] = dragCur + 3;	
									}
								}
								if (y > b) {
									if (((lastA >= 0) && (lastA < a)) || ((lastA < 0) && (x == 0))) {
										level[b+6][a] = dragCur + 1;	
									}
									if (((lastA >= 0) && (lastA > a)) || ((lastA < 0) && (x > 0))) {
										level[b+6][a] = dragCur + 4;	
									}
								}
							}
							
							level[y+6][x] = dragCur;
						}
						b = y;
						curLayer = 6;
						if ((Math.floor(level[y+6][x]/10) == Math.floor(dragCur/10)) && (level[y+6][x]%10 != 0)) {
							curLayer = 12;
							level[y+6][x] = 0;
							level[y+12][x] = dragCur;
						}
					}
				}
			}
			
			if (bLastCheck) {
				lastA = a;
				lastB = b;
				if (curLayer == 12) {
					if (((b > 0) && (b < 5) && (a == 0) && (level[y][x] != 0) && ((Math.floor(level[y][x]/2)+1) * 10 == level[y+12][x])) || ((a - 1 >= 0) && ((Math.floor(level[y+12][x-1]/10) == Math.floor(dragCur/10)) || ((Math.floor(level[y+6][x-1]/10) == Math.floor(dragCur/10)) && ((level[y+6][x-1] % 10 == 3) || (level[y+6][x-1] % 10 == 4)))))) {
						lastA = x - 1;
					} else {
						lastA = x + 1;
					}
				}
				if (curLayer == 6) {
					if (level[y+6][x]%10 == 0) {
						if (((b == 0) && (level[b][a]%2 == 1) && ((Math.floor(level[b][a]/2)+1) * 10 == level[y+6][x])) || ((b - 1 >= 0) && ((Math.floor(level[y+6-1][x]/10) == Math.floor(dragCur/10)) && ((level[y+6-1][x] % 10 == 1) || (level[y+6-1][x] % 10 == 4) || (level[y+6-1][x] % 10 == 0))))) {
							lastB = y - 1;
						} else {
							lastB = y + 1;
						}
					}
					if (level[y+6][x]%10 == 1) {
						if ((a - 1 >= 0) && ((level[y+12][x-1] == dragCur) || ((Math.floor(level[y+6][x-1]/10) == Math.floor(dragCur/10)) && ((level[y+6][x-1] % 10 == 3) || (level[y+6][x-1] % 10 == 4))))) {
							lastA = x - 1;
						} else {
							lastB = y + 1;
						}
					}
					if (level[y+6][x]%10 == 2) {
						if ((a - 1 >= 0) && ((level[y+12][x-1] == dragCur) || ((Math.floor(level[y+6][x-1]/10) == Math.floor(dragCur/10)) && ((level[y+6][x-1] % 10 == 3) || (level[y+6][x-1] % 10 == 4))))) {
							lastA = x - 1;
						} else {
							lastB = y - 1;
						}
					}
					if (level[y+6][x]%10 == 3) {
						if ((a + 1 <= 8) && ((level[y+12][x+1] == dragCur) || ((Math.floor(level[y+6][x+1]/10) == Math.floor(dragCur/10)) && ((level[y+6][x+1] % 10 == 1) || (level[y+6][x+1] % 10 == 2))))) {
							lastA = x + 1;
						} else {
							lastB = y - 1;
						}
					}
					if (level[y+6][x]%10 == 4) {
						if ((a + 1 <= 8) && ((level[y+12][x+1] == dragCur) || ((Math.floor(level[y+6][x+1]/10) == Math.floor(dragCur/10)) && ((level[y+6][x+1] % 10 == 1) || (level[y+6][x+1] % 10 == 2))))) {
							lastA = x + 1;
						} else {
							lastB = y + 1;
						}
					}
				}
				bLastCheck = false;
				bRender = true;
			}
			
			if (gMouseX < 0) {
				dragCur = b = a = -1;
				bWin = true;
				// check win
				for (j = 0; j < level.length/3; j++) {
					for (i = 0; i < level[0].length; i++) {
						if ((level[j][i] > 0) && (level[j][i] % 2 == 0)) {
							if ((level[j+6][i] == 0) && (level[j+12][i] == 0)) {
								bWin = false;
							}
							if ((level[j+6][i] != 0) && ((level[j+6][i]%10 != 0) || (level[j+6][i]/10 * 2 != level[j][i]))) {
								bWin = false;
							}
							if ((level[j+12][i] != 0) && (level[j+6][i] == 0) && (level[j+12][i]/10 * 2 != level[j][i])) {
								bWin = false;
							}
						}
					}
				}
				if (bWin) {
					bRender = true;
				}
			}
		}
	} else if (gMouseX >= 0) {
		level[0][0] = -1;
		curLevel += 1;
		if (curLevel >= levels.length / 54) {
			curLevel = 0;
		}
	}
	if (bRender) {
		Render();
		bRender = false;
	}
}

function Render(){
	// Renderabschnitt
	DrawBackground();
	
	context.font='19px bold';
	
	// tile size = 64
	context.lineWidth = 6;
	context.strokeStyle='rgb(0, 0, 0)';
	context.strokeRect(1, 64, 574, 384);
	context.lineWidth = 1;
	for (j = 0; j < level.length/3; j++) {
		for (i = 0; i < level[0].length; i++) {
			if (level[j+12][i] > 0) {
				context.fillStyle=(colors[((Math.floor)(level[j+12][i]/10)-1)*2+1]);
				context.fillRect(i * 64, (j+1) * 64 + 8, 64, 48);
			}
			if (level[j+6][i] > 0) {
				context.fillStyle=(colors[((Math.floor)(level[j+6][i]/10)-1)*2]);
				if (level[j+6][i] % 10 == 0) {
					context.fillRect(i * 64 + 8, (j+1) * 64, 48, 64);
				}
				if (level[j+6][i] % 10 == 1) {
					for (x = 0; x < 6; x++) {
						context.fillRect(i * 64 + 8 + x * 8, (j+1) * 64 + (x+1) * 8, 8, 56 - x * 8);
					}
					context.fillStyle=(colors[((Math.floor)(level[j+6][i]/10)-1)*2+1]);
					context.fillRect(i * 64, (j+1) * 64 + 8, 8, 48);
				}
				if (level[j+6][i] % 10 == 2) {
					for (x = 0; x < 6; x++) {
						context.fillRect(i * 64 + 8 + x * 8, (j+1) * 64, 8, 56 - x * 8);
					}
					context.fillStyle=(colors[((Math.floor)(level[j+6][i]/10)-1)*2+1]);
					context.fillRect(i * 64, (j+1) * 64 + 8, 8, 48);
				}
				if (level[j+6][i] % 10 == 3) {
					for (x = 0; x < 6; x++) {
						context.fillRect(i * 64 + 48 - x * 8, (j+1) * 64, 8, 56 - x * 8);
					}
					context.fillStyle=(colors[((Math.floor)(level[j+6][i]/10)-1)*2+1]);
					context.fillRect(i * 64 + 56, (j+1) * 64 + 8, 8, 48);
				}
				if (level[j+6][i] % 10 == 4) {
					for (x = 0; x < 6; x++) {
						context.fillRect(i * 64 + 48 - x * 8, (j+1) * 64 + (x+1) * 8 , 8, 56 - x * 8);
					}
					context.fillStyle=(colors[((Math.floor)(level[j+6][i]/10)-1)*2+1]);
					context.fillRect(i * 64 + 56, (j+1) * 64 + 8, 8, 48);
				}
			}
		}
	}
	
	context.lineWidth = 6;
	for (j = 0; j < level.length/3; j++) {
		for (i = 0; i < level[0].length; i++) {
			if ((level[j][i]%2 == 0) && (level[j][i] > 0)) {
				context.beginPath();
				context.fillStyle=(colors[level[j][i]-1]);
				context.strokeStyle=(colors[level[j][i]-1]);
				context.arc(i * 64 + 32, (j+1) * 64 + 32, 16, 0, Math.PI*2, true);
				context.closePath();
				if (((level[j+6][i]/10) * 2 == level[j][i]) && (level[j+6][i]%10 == 0)) {
					context.fill();
				} else if ((level[j+12][i]/10*2-1 == level[j][i] - 1) && (level[j+6][i] == 0)) {
					context.fillStyle=(colors[level[j][i] - 2]);
					context.fill();
				} else {
					context.stroke();
				}
			}
		}
	}
	context.lineWidth = 1;
	
	context.font='30px bold';
	context.fillStyle='rgb(0, 0, 0)';
	s = 'WrappingPaper4k';
	w = context.measureText(s).width;
	context.fillText(s, 288 - w/2, 50);
	
	context.strokeStyle='rgb(0, 0, 0)';
	if (!bWin) {
		context.strokeRect(273, 452, 30, 26);
		context.strokeRect(233, 452, 30, 26);
		context.strokeRect(313, 452, 30, 26);
		context.fillText('R', 288 - context.measureText('R').width/2, 475);
		context.fillText('<', 248 - context.measureText('<').width/2, 475);
		context.fillText('>', 328 - context.measureText('>').width/2, 475);
	}
	
	context.font='18px bold';
	s = 'Level: '+(curLevel+1)+'/'+(levels.length/54);
	context.fillText(s, 5, 44);
	if (bWin) {
		s = "Touch to start the next level";
		w = context.measureText(s).width;
		context.fillText(s, 288 - w/2, 475);
	}
}